function Feats = m_cmpBaoIntilleFeats(D, winSz, stepSz)
% compute feature reprentation for time-series data (esp. accelerometer data)
% The features are described in:
% Bao & Intille (2004) Activity Recognition from User-Annotated Acceleration Data,
% International conference on Pervasive Computing
% A feature vector is calculated for fixed-size (winSz frames) sliding window. 
% Consecutive windows overlap stepSz frames.
% There are four types of features for each window: DC (mean of signal), energy of FFT
% entropy of FFT, and correlation between different sensor data (different rows)
% Inputs:
%   D: d*n time series singal: column is time, d: number of sensor readings.
%   winSz: size of the sliding window
%   stepSz: distance between consecutive frames
% Ouptuts:
%   Feats: k*m matrix, where m is the number of half-overlaping windows.
%       k = d (mean) + d (energy) + d (entropy) + d*(d-1)/2 (corr) = d*(d+5)/2.
% By: Minh Hoai Nguyen (minhhoai@gmail.com)
% Date: 20 Apr 2011

[d, n] = size(D);

m = floor((n - winSz)/stepSz) + 1; % number of windows
k = d*(d+5)/2;
Feats = zeros(k, m);
for i=1:m
    startPos = 1+(i-1)*stepSz;
    endPos = startPos + winSz - 1;
    D_i = D(:, startPos:endPos); % sliding window in consideration
    
    feat_dc = mean(D_i, 2); % DC component (DC: direct current, see Wiki for DC bias)
    
    D_i_fft = abs(fft(D_i, [], 2)); % the first component is DC bias (i.e. mean value)
    % the remaining D_i_fft(:,2:end) components are symmetric by the middle column.
    D_i_fft = D_i_fft(:,2:end); % ignore the DC bias
    
    feat_energy = mean(D_i_fft.^2,2); % energy of frequency domain
    D_i_fft = D_i_fft./repmat(sum(D_i_fft,2), 1, winSz - 1); % normalize to have unit sum
    
    feat_entropy = - sum(D_i_fft.*log(D_i_fft), 2); % frequency domain entropy
    
    feat_corr = corr(D_i');
    upperMatrix = triu(ones(d, d), 1);
    feat_corr = feat_corr(logical(upperMatrix));
    
    Feats(:,i) = [feat_dc; feat_energy; feat_entropy; feat_corr];
end



 